//**************************************************************************
// tekst.c                 POGRAM ZA UREĐIVANJE TEKSTA
//**************************************************************************

#include <stdio.h>
#include <ctype.h>

//--- Parametri sistema ----------------------------------------------------
#define RED 120                    // Najveća dužina jednog reda.
#define DUZ 66                     // Početna dužina jednog reda.

//--- Simboličke konstante -------------------------------------------------
#define T   1                      // Logička konstanta "true".
#define F   0                      // Logička konstanta "false".

//--- Pregled funkcija -----------------------------------------------------
void  obradi_zn(int);              // Obrada sledećeg znaka.
_Bool smesti_zn(int);              // Smeštanje u izlazni bafer.
void  salji_red(int);              // Slanje reda na izlaz.
int   uzmi_zn(void);               // Uzimanje sledećeg znaka.
void  vrati_zn(int);               // Vraćanje jednog znaka na izlaz.
int   uzmi_broj(int, int, int);    // Uzimanje decimalnog celog broja.

//--- Globalne promenljive -------------------------------------------------
char red[RED];                     // Izlazni bafer za jedan red.
int  duz = DUZ;                    // Dužina izlaznih redova.
int  poz = 0;                      // Broj znakova u izlaznom redu.
int  rec = 0;                      // Broj reči u izlaznom redu.
char stek[RED];                    // Stek za vraćene znakove.
int  vrh = 0;                      // Broj vraćenih znakova.

//=== G l a v n a   f u n k c i j a ========================================
int main() {
  int zn;
  while ((zn = uzmi_zn()) != EOF) {
    if (isspace(zn)) zn = ' ';
    obradi_zn(zn);
 }
}
 
//=== Obrada sledećeg učitanog znaka =======================================
void obradi_zn(int zn) {
  static _Bool upr = F,            // Komanda je započeta.
               ima = T;            // Razmak je smešten.
  if (upr) {                       // Izvršavanje komande.
    switch (zn) {
    case '|':                      // Smesti znak '|'.
      ima = smesti_zn(zn); break;
    case 'P':                      // Kraj pasusa.
      salji_red(F); putchar('\n'); ima = T; break;
    case 'L':                      // Širina izlaznih redova.
      duz = uzmi_broj(1, RED, DUZ); break;
    default:                       // Nepoznata komanda.
      printf("\n*** Neispravno: |%c\n", zn); break;
    }
    upr = F;
  } else if(zn == '|') {           // Sledi komanda.
    upr = T ;
  } else if(zn != ' ' || !ima) {   // Slanje znaka.
    ima = smesti_zn(zn);
  }
}

//=== Smeštanje znaka u izlazni bafer ======================================
_Bool smesti_zn(int zn) {
  if (poz < duz) {                 // Izlazni bafer nije pun.
    red[poz++] = zn;
    if (zn == ' ') rec++;
    return zn == ' ';
  } else if (zn == ' ')            // Bafer je pun, kraj reči.
    salji_red(F);
  else {                           // Bafer je pun, nije kraj reči; ...
    if (rec) {                     // ... vrati započetu reč na ulaz ...
      while (zn != ' ') {
        vrati_zn(zn);
        zn = red[--poz];
      }
      rec--;
    }
    salji_red(T);                  // ... i salji red na izlaz.
  }
  return T;
}
 
//=== Slanje reda na izlaz uz ravnanje =====================================
void salji_red(int ravnaj) {
  // Ulaz: ravnaj - indikator ravnanja desne ivice.
  int raz,                         // Broj umetnutih razmaka između reči.
      dod,                         // Broj dodavanja po jednog razmaka.
      po,                          // Brojač pozicija pri ispisivanju.
      re,                          // Brojač reči pri ispisivanju.
      i;                           // Pomoćni brojač.

  if (ravnaj && rec) {             // Dodatni broj razmaka.
    raz = (duz - poz) / rec;
    dod = (duz - poz) % rec;
  } else {
    raz = dod = 0;
  }
  for (po=0, re=0; po<poz; po++) { // Ispisivanje.
    putchar(red[po]);
    if (red[po] == ' ') {
      for (i=0; i<raz; i++) putchar(' ');
      if (re++ < dod) putchar(' ');
    }
  }
  putchar('\n'); poz = rec = 0;
}

//=== Učitavanje sledćeg znaka =============================================
int uzmi_zn(void) { return (vrh) ? stek[--vrh] : getchar(); }

//=== Vraćanje znaka na izlaz ==============================================
void vrati_zn(int zn) { stek[vrh++] = zn; }

//=== Učitavanje decimalnog broja ==========================================
int uzmi_broj(int min, int max, int podr) {
  int zn, broj = 0;
  while (isdigit(zn = uzmi_zn())) broj = broj * 10 + zn - '0';
  if (zn != '|') vrati_zn(zn);
  if (broj < min || broj > max) {
    printf("\n***Broj izvan opsega: %d\n", broj);
    broj = podr;
  }
  return broj;
}

//**************************************************************************
